@@ -1,4 +1,4 @@
-static const char _findif_c [] = "$Id: findif.c,v 1.22.2.5 2003/08/19 14:21:53 lars Exp $";
+/* $Id: findif.c,v 1.32.2.2 2004/04/29 01:28:07 alan Exp $ */
 /*
  * findif.c:	Finds an interface which can route a given address
  *
@@ -91,9 +91,9 @@ static const char _findif_c [] = "$Id: findif.c,v 1.22.2.5 2003/08/19 14:21:53 l
 
 int	OutputInCIDR=0;
 
-void ConvertQuadToInt (char *dest);
+void ConvertQuadToInt (char *dest, int destlen);
 
-void ConvertBitsToMask (char *mask);
+void ConvertBitsToMask (char *mask, int masklen);
 
 /*
  * Different OSes offer different mechnisms to obtain this information.
@@ -108,11 +108,12 @@ void ConvertBitsToMask (char *mask);
  *	On non-zero, errmsg may have been filled with an error message
  */
 typedef int SearchRoute (char *address, struct in_addr *in
-,        struct in_addr *addr_out, char *best_if
-,        unsigned long *best_netmask, char *errmsg);
+,        struct in_addr *addr_out, char *best_if, size_t best_iflen
+,        unsigned long *best_netmask, char *errmsg
+,	int errmsglen);
 
-SearchRoute SearchUsingProcRoute;
-SearchRoute SearchUsingRouteCmd;
+static SearchRoute SearchUsingProcRoute;
+static SearchRoute SearchUsingRouteCmd;
 
 SearchRoute *search_mechs[] = {
 	&SearchUsingProcRoute,
@@ -133,7 +134,7 @@ void usage(void);
 #define	MAXSTR	128
 
 void
-ConvertQuadToInt (char *dest)
+ConvertQuadToInt (char *dest, int destlen)
 {
 	unsigned ipquad[4] = { 0, 0, 0, 0 };
 	unsigned long int intdest = 0;
@@ -166,11 +167,11 @@ ConvertQuadToInt (char *dest)
 	|		((ipquad[2]&0xff) <<8) 
 	|	 	 (ipquad[3]&0xff));
 #endif
-	sprintf (dest, "%ld", intdest);
+	snprintf (dest, destlen, "%ld", intdest);
 }
 
 void
-ConvertBitsToMask (char *mask)
+ConvertBitsToMask (char *mask, int masklen)
 {
 	int maskbits;
 	unsigned long int maskflag = 0x1;
@@ -178,6 +179,7 @@ ConvertBitsToMask (char *mask)
 	int i;
 	int count;
 	char *p;
+	char *lastp=mask+masklen;
 	char maskwithoutdots[12];
 
 	/*
@@ -185,14 +187,14 @@ ConvertBitsToMask (char *mask)
 	 * (like from the output of the "route get" command)
 	 */
 	if ((p = strtok(mask, ".")) != NULL) {
-		i = strlen(p);
+		i = strnlen(p, masklen);
 
 		strncpy(maskwithoutdots, p, (i <= 3 ? i : 3));
 
 		for (count=0; count < 3; count++) {
 			p=strtok(NULL, ".");
 			if (p != NULL) {
-				i = strlen(p);
+				i = strnlen(p, lastp-p);
 				strncat(maskwithoutdots, p
 				,	(i <= 3 && p != NULL ? i : 3));
 			}
@@ -216,23 +218,26 @@ ConvertBitsToMask (char *mask)
 	}
 
 	/* printf ("longmask: %u (%X)\n", longmask, longmask); */
-	sprintf (mask, "%ld", longmask);
+	snprintf (mask, masklen, "%ld", longmask);
 }
 
-int
+static int
 SearchUsingProcRoute (char *address, struct in_addr *in
-, 	 struct in_addr *addr_out, char *best_if
-,	 unsigned long *best_netmask, char *errmsg)
+, 	struct in_addr *addr_out, char *best_if, size_t best_iflen
+,	unsigned long *best_netmask
+,	char *errmsg, int errmsglen)
 {
-	long    flags, refcnt, use, metric, dest, gw, mask;
-	long	best_metric = LONG_MAX;
+	long    	flags, refcnt, use, metric, gw, mask;
+	unsigned long   dest;
+	long		best_metric = LONG_MAX;
 	
 	char	buf[2048];
 	char	interface[MAXSTR];
 	FILE *routefd = NULL;
 
 	if ((routefd = fopen(PROCROUTE, "r")) == NULL) {
-		sprintf(errmsg, "Cannot open %s for reading"
+		snprintf(errmsg, errmsglen
+		,	"Cannot open %s for reading"
 		,	PROCROUTE);
 		return(-1);
 	}
@@ -244,7 +249,7 @@ SearchUsingProcRoute (char *address, struct in_addr *in
 		,	interface, &dest, &gw, &flags, &refcnt, &use
 		,	&metric, &mask)
 		!= 8) {
-			sprintf(errmsg, "Bad line in %s: %s"
+			snprintf(errmsg, errmsglen, "Bad line in %s: %s"
 			,	PROCROUTE, buf);
 			return(1);
 		}
@@ -252,23 +257,24 @@ SearchUsingProcRoute (char *address, struct in_addr *in
 		&&	metric < best_metric) {
 			best_metric = metric;
 			*best_netmask = mask;
-			strcpy(best_if, interface);
+			strncpy(best_if, interface, best_iflen);
 		}
 	}
 	fclose(routefd);
 
 	if (best_metric == LONG_MAX) {
-		sprintf(errmsg, "No route to %s\n", address);
+		snprintf(errmsg, errmsglen, "No route to %s\n", address);
 		return(1); 
 	}
 
 	return(0);
 }
 
-int
+static int
 SearchUsingRouteCmd (char *address, struct in_addr *in
-,	 struct in_addr *addr_out, char *best_if
-,	 unsigned long *best_netmask, char *errmsg)
+,	struct in_addr *addr_out, char *best_if, size_t best_iflen
+,	unsigned long *best_netmask
+,	char *errmsg, int errmsglen)
 {
 	char	dest[20], mask[20];
 	char	routecmd[MAXSTR];
@@ -281,25 +287,27 @@ SearchUsingRouteCmd (char *address, struct in_addr *in
 
 	
 	/* Open route and get the information */
-	sprintf (routecmd, "%s %s %s", ROUTE, ROUTEPARM, address);
+	snprintf (routecmd, errmsglen, "%s %s %s", ROUTE, ROUTEPARM, address);
 	routefd = popen (routecmd, "r");
 	if (routefd == NULL)
 		return (-1);
 	mask[0] = EOS;
 
+
 	while ((done < 3) && fgets(buf, sizeof(buf), routefd)) {
+		int buflen = strnlen(buf, sizeof(buf));
 		cp = buf;
 
-		sp = buf + strlen(buf);
+		sp = buf + buflen;
 		while (sp!=buf && isspace((int)*(sp-1))) --sp;
 		*sp = '\0';
 
-		buf[strlen(buf)] = EOS;
+		buf[buflen] = EOS;
 		if (strstr (buf, "mask:")) {
 			/*strsep(&cp, ":");cp++;*/
 			cp = strtok(buf, ":");
 			cp = strtok(NULL, ":");cp++;
-			strcpy(mask, cp);
+			strncpy(mask, cp, sizeof(mask));
                   	done++;
 		}
 
@@ -307,7 +315,7 @@ SearchUsingRouteCmd (char *address, struct in_addr *in
 			/*strsep(&cp, ":");cp++;*/
 			cp = strtok(buf, ":");
 			cp = strtok(NULL, ":");cp++;
-			strcpy(interface, cp);
+			strncpy(interface, cp, sizeof(interface));
                   	done++;
 		}
 
@@ -315,7 +323,7 @@ SearchUsingRouteCmd (char *address, struct in_addr *in
 			/*strsep(&cp, ":");cp++;*/
 			cp = strtok(buf, ":");
 			cp = strtok(NULL, ":");cp++;
-			strcpy(dest, cp);
+			strncpy(dest, cp, sizeof(dest));
                   	done++;
 		}
 	}
@@ -332,17 +340,18 @@ SearchUsingRouteCmd (char *address, struct in_addr *in
 	 * return packets to the originator.  Without the right subnet
 	 * mask, you can't reply to any packets you receive.
 	 */
-	if (strlen(mask) == 0) {
-		strcpy (mask, "255.255.255.255");
+	if (strnlen(mask, sizeof(mask)) == 0) {
+		strncpy (mask, "255.255.255.255", sizeof(mask));
 	}
-	ConvertQuadToInt  (dest);
-	ConvertBitsToMask (mask);
+	ConvertQuadToInt  (dest, sizeof(dest));
+	ConvertBitsToMask (mask, sizeof(mask));
 /*
 	ConvertMaskToInt (mask);
 */
 
 	if (inet_pton(AF_INET, address, addr_out) <= 0) {
-		sprintf(errmsg, "IP address [%s] not valid.", address);
+		snprintf(errmsg, errmsglen
+		,	"IP address [%s] not valid.", address);
 		usage();
 		return(1);
 	}
@@ -350,12 +359,12 @@ SearchUsingRouteCmd (char *address, struct in_addr *in
 	if ((in->s_addr & atoi(mask)) == (addr_out->s_addr & atoi(mask))) {
 		best_metric = 0;
 		*best_netmask = atoi(mask);
-		strcpy(best_if, interface);
+		strncpy(best_if, interface, best_iflen);
 	}
 
 	fclose(routefd);
 	if (best_metric == INT_MAX) {
-		sprintf(errmsg, "No route to %s\n", address);
+		snprintf(errmsg, errmsglen, "No route to %s\n", address);
 		return(1);
 	}
 
@@ -402,9 +411,12 @@ void
 ValidateNetmaskBits (char *netmaskbits, unsigned long *netmask)
 {
 	if (netmaskbits != NULL) {
-		if ((strspn(netmaskbits, "0123456789")
-		!=	strlen(netmaskbits)
-		||	strlen(netmaskbits) == 0)) {
+		size_t	nmblen = strnlen(netmaskbits, 3);
+
+		/* Maximum netmask is 32 */
+
+		if (nmblen > 2 || nmblen == 0
+		||	(strspn(netmaskbits, "0123456789") != nmblen)) {
 			fprintf(stderr, "Invalid netmask specification"
 			" [%s]", netmaskbits);
 			usage();
@@ -429,7 +441,7 @@ ValidateNetmaskBits (char *netmaskbits, unsigned long *netmask)
 int
 main(int argc, char ** argv) {
 
-	char *	iparg;
+	char *	iparg = NULL;
 	char *	address = NULL;
 	char *	bcast_arg = NULL;
 	char *	netmaskbits = NULL;
@@ -437,13 +449,10 @@ main(int argc, char ** argv) {
 	struct in_addr	addr_out;
 	unsigned long	netmask;
 	char	best_if[MAXSTR];
-	char	output_netmask[MAXSTR];
-	char	output_bcast[MAXSTR];
 	char *	if_specified = NULL;
 	unsigned long	best_netmask = INT_MAX;
 	int		argerrs	= 0;
 
-	(void)_findif_c;
 	cmdname=argv[0];
 
 
@@ -458,7 +467,7 @@ main(int argc, char ** argv) {
 		iparg=argv[1];
 		break;
 	case 3: /* Hopefully a -C argument */
-		if (strcmp(argv[1], "-C") != 0) {
+		if (strncmp(argv[1], "-C", sizeof("-C")) != 0) {
 			argerrs=1;
 		}
 		OutputInCIDR=1;
@@ -488,7 +497,8 @@ main(int argc, char ** argv) {
 	ValidateNetmaskBits (netmaskbits, &netmask);
 
 	if (if_specified != NULL) {
-		strcpy(best_if, if_specified);
+		strncpy(best_if, if_specified, sizeof(best_if));
+		*(best_if + sizeof(best_if) - 1) = '\0';
 	}else{
 		SearchRoute **sr = search_mechs;
 		char errmsg[MAXSTR] = "No valid mecahnisms";
@@ -497,15 +507,17 @@ main(int argc, char ** argv) {
 		while (*sr) {
 			errmsg[0] = '\0';
 			rc = (*sr) (address, &in, &addr_out, best_if
-			,	&best_netmask, errmsg);
+			,	sizeof(best_if)
+			,	&best_netmask, errmsg, sizeof(errmsg));
 			if (rc >= 0) {		/* Mechanism worked */
 				break;
 			}
 			sr++;
 		}
-		if (rc != 0) {		/* No route, or all mechanisms failed */
+		if (rc != 0) {	/* No route, or all mechanisms failed */
 			if (*errmsg) {
 				fprintf(stderr, "%s", errmsg);
+				return(1);
 			}
 		}
 	}
@@ -518,11 +530,21 @@ main(int argc, char ** argv) {
 		,	 address);
 		return(1);
 	}
-	
+		
+
+
 	/* Did they tell us the broadcast address? */
+
 	if (bcast_arg) {
-		/* Yes, specified on the commandline, no validation */
-		strcpy(output_bcast,bcast_arg);
+		best_netmask = htonl(best_netmask);
+		/* Yes, they gave us a broadcast address */
+		printf("%s\t netmask %d.%d.%d.%d\tbroadcast %s\n"
+		,	best_if
+                ,       (int)((best_netmask>>24) & 0xff)
+                ,       (int)((best_netmask>>16) & 0xff)
+                ,       (int)((best_netmask>>8) & 0xff)
+                ,       (int)(best_netmask & 0xff)
+		,	bcast_arg);
 	}else{
 		/* No, we use a common broadcast address convention */
 		unsigned long	def_bcast;
@@ -535,39 +557,38 @@ main(int argc, char ** argv) {
 #endif
 
 		/* Make things a bit more machine-independent */
+		best_netmask = htonl(best_netmask);
 		def_bcast = htonl(def_bcast);
-
-		sprintf(output_bcast, "%d.%d.%d.%d"
+		if (!OutputInCIDR) {
+			printf("%s\tnetmask %d.%d.%d.%d\tbroadcast %d.%d.%d.%d\n"
+			,       best_if
+			,       (int)((best_netmask>>24) & 0xff)
+			,       (int)((best_netmask>>16) & 0xff)
+			,       (int)((best_netmask>>8) & 0xff)
+			,       (int)(best_netmask & 0xff)
 			,       (int)((def_bcast>>24) & 0xff)
 			,       (int)((def_bcast>>16) & 0xff)
 			,       (int)((def_bcast>>8) & 0xff)
 			,       (int)(def_bcast & 0xff));
-	}
+		}else{
+			int	j;
 
-	best_netmask = htonl(best_netmask);
-	/* Convert netmask for output */
-	if (!OutputInCIDR) {
-		sprintf(output_netmask, "%d.%d.%d.%d"
-		,       (int)((best_netmask>>24) & 0xff)
-		,       (int)((best_netmask>>16) & 0xff)
-		,       (int)((best_netmask>>8) & 0xff)
-		,       (int)(best_netmask & 0xff));
-	}else{
-		int	j;
+			best_netmask = best_netmask & 0xFFFFFFFFUL;
 
-		best_netmask = best_netmask & 0xFFFFFFFFUL;
-
-		for (j=0; j <= 32; ++j) {
-			if ((best_netmask >> j)&0x1) {
-				break;
+			for (j=0; j <= 32; ++j) {
+				if ((best_netmask >> j)&0x1) {
+					break;
+				}
 			}
+			printf("%s\tnetmask %d\tbroadcast %d.%d.%d.%d\n"
+			,       best_if
+			,	32-j
+			,       (int)((def_bcast>>24) & 0xff)
+			,       (int)((def_bcast>>16) & 0xff)
+			,       (int)((def_bcast>>8) & 0xff)
+			,       (int)(def_bcast & 0xff));
 		}
-		sprintf(output_netmask, "%d", 32-j);
 	}
-
-	printf("%s\tnetmask %s\tbroadcast %s\n"
-	      ,	best_if, output_netmask, output_bcast);
-	
 	return(0);
 }
 
@@ -658,23 +679,45 @@ ff02::%lo0/32                     fe80::1%lo0                   UC          lo0
 
 /* 
  * $Log: findif.c,v $
- * Revision 1.22.2.5  2003/08/19 14:21:53  lars
- * Fix findif output for CIDR notation if broadcast address is specified on
- * the commandline.
+ * Revision 1.32.2.2  2004/04/29 01:28:07  alan
+ * Put in a small fix to findif provided by Lars Ellenberg.
  *
- * Revision 1.22.2.4  2003/08/12 21:17:46  alan
- * Moved a fix over from unstable for netmask mangling from Tuomo.
+ * Revision 1.32.2.1  2004/04/20 05:05:27  alan
+ * Backported everything from 1.3.0 to 1.2.1
  *
- * Revision 1.22.2.3  2003/06/18 03:39:00  horms
+ * Revision 1.34  2004/03/25 07:55:39  alan
+ * Moved heartbeat libraries to the lib directory.
+ *
+ * Revision 1.33  2004/02/17 22:11:57  lars
+ * Pet peeve removal: _Id et al now gone, replaced with consistent Id header.
+ *
+ * Revision 1.32  2004/01/20 10:16:08  horms
+ * Should be strncpy not strncmp
+ *
+ * Revision 1.31  2004/01/14 17:51:07  alan
+ * Put in some minor code cleanups suggested by Emily Ratliff.
+ *
+ * Revision 1.30  2003/09/26 05:55:54  alan
+ * Fixed an undefined variable warning...
+ *
+ * Revision 1.29  2003/07/02 22:27:00  alan
+ * Fixed a bug in the broadcast output format for CIDR -C option for findif.
+ *
+ * Revision 1.28  2003/06/18 03:43:42  horms
  * spelling
  *
- * Revision 1.22.2.2  2003/06/02 11:06:09  lars
- * Sync with CVS head in preparation for 1.0.3:
- * - Should now work correctly with a /32 netmask
- * - Minor cleanups
+ * Revision 1.27  2003/04/19 03:59:15  alan
+ * Put in an enhancement to findif to allow Tuomo Soini to get the output he
+ * wants in CIDR format.
  *
- * Revision 1.22.2.1  2003/03/12 18:24:50  lars
- * Syncing 1.0.x series with CVS head in preparation for 1.0.2 release.
+ * Revision 1.26  2003/04/15 01:58:54  horms
+ * turn debuging off
+ *
+ * Revision 1.25  2003/04/14 10:22:44  horms
+ * should now work correclty with /32
+ *
+ * Revision 1.24  2003/04/14 09:53:17  horms
+ * minor reformating
  *
  * Revision 1.23  2003/02/27 06:46:35  horms
  * Slightly better documentation of the options to findif and thus IPaddr.
