@@ -1,4 +1,4 @@
-static const char * _ha_msg_c_Id = "$Id: ha_msg_internal.c,v 1.32 2003/02/07 08:37:16 horms Exp $";
+/* $Id: ha_msg_internal.c,v 1.39.2.1 2004/04/20 05:05:27 alan Exp $ */
 /*
  * ha_msg_internal: heartbeat internal messaging functions
  *
@@ -31,151 +31,19 @@ static const char * _ha_msg_c_Id = "$Id: ha_msg_internal.c,v 1.32 2003/02/07 08:
 #include <sys/utsname.h>
 #include <heartbeat.h>
 #include <ha_msg.h>
+#include <heartbeat_private.h>
 
-#define		MINFIELDS	20
+#define		MINFIELDS	30
 #define		CRNL		"\r\n"
 
-/* Return the next message found in the stream and copies */
-/* the iface in "iface"  */
 
-struct ha_msg *
-if_msgfromstream(FILE * f, char *iface)
-{
-	char		buf[MAXLINE];
-	const char *	bufmax = buf + sizeof(buf);
-	char *		getsret;
-	struct ha_msg*	ret;
-	
-
-	(void)_ha_msg_c_Id;
-	(void)_heartbeat_h_Id;
-	(void)_ha_msg_h_Id;
-	clearerr(f);
-
-	if(!(getsret=fgets(buf, MAXLINE, f))) { 
-		if (!ferror(f) || errno != EINTR) 
-			ha_log(LOG_ERR, "if_msgfromstream: cannot get message");
-		return(NULL);
-	}
-
-	/* Try to find the interface in the message. */
-
-	if (!strcmp(buf, IFACE)) {
-		/* Found interface name header, get interface name. */
-		if(!(getsret=fgets(buf, MAXLINE, f))) { 
-			if (!ferror(f) || errno != EINTR)
-				ha_log(LOG_ERR, "if_msgfromstream: "
-						"cannot get message");
-			return(NULL);
-		}
-		if (iface) { 
-			int len = strlen(buf);
-			if(len < MAXIFACELEN) {
-				strncpy(iface, buf, len);
-				iface[len -1] = EOS;
-			}
-		}
-	}
-
-	if (strcmp(buf, MSG_START)) { 	
-		/* Skip until we find a MSG_START (hopefully we skip nothing) */
-		while ((getsret=fgets(buf, MAXLINE, f)) != NULL
-		&&	strcmp(buf, MSG_START) != 0) {
-			/* Nothing */
-		}
-	}
-
-	if (getsret == NULL || (ret = ha_msg_new(0)) == NULL) {
-		/* Getting an error with EINTR is pretty normal */
-		if (!ferror(f) || errno != EINTR) {
-			ha_log(LOG_ERR, "if_msgfromstream: "
-					"cannot get message");
-		}
-		return(NULL);
-	}
-
-	/* Add Name=value pairs until we reach MSG_END or EOF */
-	while ((getsret=fgets(buf, MAXLINE, f)) != NULL
-	&&	strcmp(buf, MSG_END) != 0) {
-
-		/* Add the "name=value" string on this line to the message */
-		if (ha_msg_add_nv(ret, buf, bufmax) != HA_OK) {
-			ha_log(LOG_INFO
-			,	"NV failure (if_msgfromsteam)(%s): [%s]"
-			,	iface, buf);
-			ha_msg_del(ret);
-			return(NULL);
-		}
-	}
-	return(ret);
-}
-
-/*
- *	Output string encoding both message and interface it came in on.
- */
-char *
-msg2if_string(const struct ha_msg *m, const char *iface) 
-{
-
-	int	j;
-	char *	buf;
-	char *	bp;	/* current position in output string (buf)
-			 * Maintaining this makes this code lots faster because
-			 * otherwise strcat is pretty slow
-			 */
-	int	ifaceLen;
-	int	mlen;
-
-	if (m->nfields <= 0) {
-		ha_log(LOG_ERR, "msg2if_string: Message with zero fields");
-		return(NULL);
-	}
-
-	ifaceLen = strlen(iface);
-
-	/* Note: m->stringlen is # of chars to convert "m" to a plain string */
-	mlen = STRLEN(IFACE) + ifaceLen + STRLEN("\n") + m->stringlen;
-
-	buf = ha_malloc(mlen * sizeof(char ));
-
-	if (buf == NULL) {
-		ha_log(LOG_ERR, "msg2if_string: no memory for string");
-	}else{
-		/* Prepend information indicating incoming "interface" */
-		strcpy(buf, IFACE);
-		bp = buf + STRLEN(IFACE);
-
-		strcat(bp, iface);
-		bp += ifaceLen;
-
-		strcat(bp, "\n");
-		bp += STRLEN("\n");
-
-		/* Append the normal (plain) string representation of the message */
-		strcat(buf, MSG_START);
-		for (j=0; j < m->nfields; ++j) {
-
-			strcat(bp, m->names[j]);
-			bp += m->nlens[j];
-
-			strcat(bp, "=");
-			bp += STRLEN("=");
-
-			strcat(bp, m->values[j]);
-			bp += m->vlens[j];
-
-			strcat(bp, "\n");
-			bp += STRLEN("\n");
-		}
-		strcat(bp, MSG_END);
-	}
-	return(buf);
-}
 
 
 #define	SEQ	"seq"
 #define	LOAD1	"load1"
 
+extern int		netstring_format;
+
 /* The value functions are expected to return pointers to static data */
 struct default_vals {
 	const char *	name;
@@ -200,75 +68,23 @@ struct default_vals defaults [] = {
 	{F_TTL,		ha_msg_ttl, 0},
 };
 
-/* Reads from control fifo, and creates a new message from it */
-/* (this adds a few default fields with timestamp, sequence #, etc.) */
 struct ha_msg *
-controlfifo2msg(FILE * f)
+add_control_msg_fields(struct ha_msg* ret)
 {
-	char		buf[MAXLINE];
-	char		garbbuf[MAXLINE];
-	const char *	bufmax = buf + sizeof(buf);
-	char *		getsret;
-	const char*	type;
-	struct ha_msg*	ret;
+	const char *	type;
 	int		j;
 	int		noseqno;
 
-
-	garbbuf[0] = '\0';
-	/* Skip until we find a MSG_START (hopefully we skip nothing) */
-	while ((getsret=fgets(buf, MAXLINE, f)) != NULL
-	&&	strcmp(buf, MSG_START) != 0) {
-		/* Save this for travkin WHAT we got. */
-		/* Nothing */
-	}
-
-	if (getsret == NULL || (ret = ha_msg_new(0)) == NULL) {
-		return(NULL);
-	}
-
-	sprintf(garbbuf, "%s ", buf);
-	/* Add Name=value pairs until we reach MSG_END or EOF */
-	while ((getsret=fgets(buf, MAXLINE, f)) != NULL
-	&&	strcmp(buf, MSG_END) != 0) {
-
-		sprintf(garbbuf, "%s %s", garbbuf, buf);
-		/* This shouldn't happen but it does on FreeBSD! */
-		/* FIXME (FreeBSD gets a newline and blank lines) !! */
-		if ((*buf == '\n') || (*buf == EOS)) {
-			continue;
-		}
-
-		/* Add the "name=value" string on this line to the message */
-		if (ha_msg_add_nv(ret, buf, bufmax) != HA_OK) {
-			ha_log(LOG_ERR, "NV failure (controlfifo2msg):");
-			ha_log(LOG_DEBUG, "[%s] %ld chars", buf,
-				(long)strlen(buf));
-			ha_log(LOG_DEBUG, "Read in message    : '%s'",
-				garbbuf);
-			ha_log_message(ret);
-			ha_msg_del(ret);
-			return(NULL);
-		}
-	}
-
-	if (DEBUGPKTCONT) {
-		sprintf(garbbuf, "%s %s", garbbuf, buf);
-		ha_log(LOG_DEBUG, "Read in message    : '%s'", garbbuf);
-	}
-
 	if ((type = ha_msg_value(ret, F_TYPE)) == NULL) {
-		ha_log(LOG_ERR, "No type (controlfifo2msg): ");
-		ha_log(LOG_DEBUG, "[%s] %ld chars", buf, (long)strlen(buf));
-		ha_log(LOG_DEBUG, "Read in message    : '%s'", garbbuf);
-		ha_log_message(ret);
+		ha_log(LOG_ERR, "No type (add_control_msg_fields): ");
+		cl_log_message(ret);
 		ha_msg_del(ret);
 		return(NULL);
 	}
 
 	if (DEBUGPKTCONT) {
-		ha_log(LOG_DEBUG, "controlfifo2msg: input packet");
-		ha_log_message(ret);
+		ha_log(LOG_DEBUG, "add_control_msg_fields: input packet");
+		cl_log_message(ret);
 	}
 
 	noseqno = (strncmp(type, NOSEQ_PREFIX, sizeof(NOSEQ_PREFIX)-1) == 0);
@@ -299,28 +115,27 @@ controlfifo2msg(FILE * f)
 			ha_msg_del(ret);
 			return(NULL);
 		}
-	}
-	if (!add_msg_auth(ret)) {
+	} 
+	if (!netstring_format && !add_msg_auth(ret)) {
 		ha_msg_del(ret);
 		ret = NULL;
 	}
 	if (DEBUGPKTCONT) {
-		ha_log(LOG_DEBUG, "controlfifo2msg: packet returned");
-		ha_log_message(ret);
+		ha_log(LOG_DEBUG, "add_control_msg_fields: packet returned");
+		cl_log_message(ret);
 	}
-
-	return(ret);
+	return ret;
 }
 
+
+
 int
 add_msg_auth(struct ha_msg * m)
 {
 	char	msgbody[MAXMSG];
 	char	authstring[MAXLINE];
 	char	authtoken[MAXLINE];
-	char *	bp = msgbody;
-	int	j;
-
+	
 	{
 		const char *	from;
 		const char *	ts;
@@ -336,29 +151,23 @@ add_msg_auth(struct ha_msg * m)
 			,	"add_msg_auth: %s:  from %s"
 			,	"missing from/ts/type"
 			,	(from? from : "<?>"));
-			ha_log_message(m);
+			cl_log_message(m);
 		}
 	}
 
 	check_auth_change(config);
 	msgbody[0] = EOS;
-	for (j=0; j < m->nfields; ++j) {
-		/* Skip over any F_AUTH fields we find... */
-		if (strcmp(m->names[j], F_AUTH) == 0) {
-			continue;
-		}
-		strcat(bp, m->names[j]);
-		bp += m->nlens[j];
-		strcat(bp, "=");
-		bp++;
-		strcat(bp, m->values[j]);
-		bp += m->vlens[j];
-		strcat(bp, "\n");
-		bp++;
+	
+	if (msg2string_buf(m, msgbody, MAXMSG, 0, NOHEAD) != HA_OK){
+		ha_log(LOG_ERR
+		       ,	"add_msg_auth: compute string failed");
+		return(HA_FAIL);
 	}
+	
 
-
+	
 	if (!config->authmethod->auth->auth(config->authmethod, msgbody
+	,	strnlen(msgbody, MAXMSG)
 	,	authtoken, DIMOF(authtoken))) {
 		ha_log(LOG_ERR 
 		,	"Cannot compute message authentication [%s/%s/%s]"
@@ -375,44 +184,42 @@ add_msg_auth(struct ha_msg * m)
 	return(ha_msg_mod(m, F_AUTH, authstring));
 }
 
-int
+gboolean
 isauthentic(const struct ha_msg * m)
 {
 	char	msgbody[MAXMSG];
 	char	authstring[MAXLINE];
 	char	authbuf[MAXLINE];
 	const char *	authtoken = NULL;
-	char *	bp = msgbody;
 	int	j;
 	int	authwhich = 0;
 	struct HBauth_info*	which;
 	
+	
 	if (m->stringlen >= sizeof(msgbody)) {
 		return(0);
 	}
-
+	
 	/* Reread authentication? */
 	check_auth_change(config);
 
-	msgbody[0] = EOS;
+
+	if (msg2string_buf(m, msgbody, MAXMSG,0, NOHEAD) != HA_OK){
+		ha_log(LOG_ERR
+		       ,	"add_msg_auth: compute string failed");
+		return(HA_FAIL);
+	}
+	
 	for (j=0; j < m->nfields; ++j) {
 		if (strcmp(m->names[j], F_AUTH) == 0) {
 			authtoken = m->values[j];
 			continue;
-		}
-		strcat(bp, m->names[j]);
-		bp += m->nlens[j];
-		strcat(bp, "=");
-		bp++;
-		strcat(bp, m->values[j]);
-		bp += m->vlens[j];
-		strcat(bp, "\n");
-		bp++;
-	}
+		}		
+	}	
 	
 	if (authtoken == NULL
 	||	sscanf(authtoken, "%d %s", &authwhich, authstring) != 2) {
-		ha_log(LOG_WARNING, "Bad/invalid auth token");
+		ha_log(LOG_WARNING, "Bad/invalid auth token, authtoken=%p", authtoken);
 		return(0);
 	}
 	which = config->auth_config + authwhich;
@@ -425,7 +232,9 @@ isauthentic(const struct ha_msg * m)
 	}
 		
 	
-	if (!which->auth->auth(which, msgbody, authbuf, DIMOF(authbuf))) {
+	if (!which->auth->auth(which
+        ,	msgbody, strnlen(msgbody, MAXMSG)
+	,	authbuf, DIMOF(authbuf))) {
 		ha_log(LOG_ERR, "Failed to compute message authentication");
 		return(0);
 	}
@@ -436,7 +245,8 @@ isauthentic(const struct ha_msg * m)
 		return(1);
 	}
 	if (DEBUGAUTH) {
-		ha_log(LOG_INFO, "Packet failed authentication check");
+		ha_log(LOG_INFO, "Packet failed authentication check, "
+		       "authstring =%s,authbuf=%s ", authstring, authbuf);
 	}
 	return(0);
 }
@@ -446,13 +256,7 @@ isauthentic(const struct ha_msg * m)
 STATIC	const char *
 ha_msg_from(void)
 {
-	static struct utsname u;
-	static int uyet = 0;
-	if (!uyet) {
-		uname(&u);
-		uyet++;
-	}
-	return(u.nodename);
+	return localnodename;
 }
 
 /* Add sequence number field */
@@ -471,7 +275,7 @@ STATIC	const char *
 ha_msg_timestamp(void)
 {
 	static char ts[32];
-	sprintf(ts, "%lx", time(NULL));
+	sprintf(ts, TIME_X, (TIME_T)time(NULL));
 	return(ts);
 }
 
@@ -548,6 +352,53 @@ main(int argc, char ** argv)
 #endif
 /*
  * $Log: ha_msg_internal.c,v $
+ * Revision 1.39.2.1  2004/04/20 05:05:27  alan
+ * Backported everything from 1.3.0 to 1.2.1
+ *
+ * Revision 1.42  2004/03/25 07:55:39  alan
+ * Moved heartbeat libraries to the lib directory.
+ *
+ * Revision 1.41  2004/03/03 05:31:50  alan
+ * Put in Gochun Shi's new netstrings on-the-wire data format code.
+ * this allows sending binary data, among many other things!
+ *
+ * Revision 1.40  2004/02/17 22:11:57  lars
+ * Pet peeve removal: _Id et al now gone, replaced with consistent Id header.
+ *
+ * Revision 1.39  2003/12/19 17:28:01  alan
+ * Incorporated a patch from Guochun Shi <gshi@ncsa.uiuc.edu> to allow
+ * the authentication facility to be able to authenticate binary messages.
+ *
+ * Revision 1.38  2003/11/10 08:55:20  lars
+ * Bugfixes by Deng, Pan:
+ *
+ * - While receiving a ha_msg, the default number of fields is MINFIELDS,
+ *   which is 20. After the reception, if more than 20 fields needed to be
+ *   added, it will fail.  I changed the MINFIELDS to 30. It is not a
+ *   graceful fix, but it can work for checkpoint service. I think the max
+ *   fields should not be fixed.
+ *
+ * - The message create routine ha_msg_new() in ha_msg.c. It takes a
+ *   parameter nfields, but the function does not use it at all. If nfields
+ *   > MINFIELDS, the allocated fields should be nfields.
+ *
+ * Revision 1.37  2003/07/03 21:50:40  alan
+ * Changed the function which adds the "from" address to not use uname, but
+ * the already-computed localnodename variable.
+ *
+ * Revision 1.36  2003/05/30 15:22:11  kevin
+ * Fix building on OpenBSD
+ *
+ * Revision 1.35  2003/04/18 06:09:46  alan
+ * Fixed an off-by-one error in writing messages to the FIFO.
+ * Also got rid of some now-unused functions, and fixed a minor glitch in BasicSanitCheck.
+ *
+ * Revision 1.34  2003/04/15 23:06:53  alan
+ * Lots of new code to support the semi-massive process restructuriing.
+ *
+ * Revision 1.33  2003/03/29 02:48:44  alan
+ * More small changes on the road to restructuring heartbeat processees.
+ *
  * Revision 1.32  2003/02/07 08:37:16  horms
  * Removed inclusion of portability.h from .h files
  * so that it does not need to be installed.
