@@ -1,3 +1,4 @@
+/* $Id: hb_api.h,v 1.12.2.1 2004/04/20 07:57:54 alan Exp $ */
 /*
  * Client-side Low-level clustering API for heartbeat.
  *
@@ -44,6 +45,7 @@
 #ifndef __HB_API_H
 #	define __HB_API_H 1
 #include <ha_msg.h>
+#include <clplumbing/ipc.h>
 
 #define	LLC_PROTOCOL_VERSION	1
 
@@ -57,6 +59,10 @@ typedef void (*llc_ifstatus_callback_t) (const char *node
 ,	const char * interface, const char * status
 ,	void* private_data);
 
+typedef void (*llc_cstatus_callback_t) (const char *node
+,	const char * client, const char * status
+,	void* private_date);
+
 typedef struct ll_cluster {
 	void *		ll_cluster_private;
 	struct llc_ops*	llc_ops;
@@ -109,8 +115,20 @@ struct llc_ops {
  *	p:		private data - later passed to callback.
  */
 	int             (*set_ifstatus_callback) (ll_cluster_t*
-,			llc_ifstatus_callback_t cbf, void * p);
- 
+	,		llc_ifstatus_callback_t cbf, void * p);
+/*
+ *	set_cstatus_callback:	Define callback from client status messages
+ *				This is a message of type "hbapi-clstat"
+ *			These messages are received whenever an client on
+ *			other nodes goes dead or becomes active again.
+ *
+ *	cbf		callback function.
+ *
+ *	p:		private data - later passed to callback.
+ */
+	int		(*set_cstatus_callback) (ll_cluster_t*
+	,		llc_cstatus_callback_t cbf, void * p);
+
 
 /*************************************************************************
  * Getting Current Information
@@ -153,6 +171,11 @@ struct llc_ops {
  */
 	const char*	(*if_status)(ll_cluster_t*, const char * nodename
 ,			const char *iface);
+/*
+ *	client_status:	Return current status of the given client
+ */
+	const char*	(*client_status)(ll_cluster_t*, const char *host,
+			const char *clientid, int timeout);
 
 /*************************************************************************
  * Intracluster messaging
@@ -170,12 +193,39 @@ struct llc_ops {
 ,			struct ha_msg* msg
 ,			const char * nodename);
 
+/*
+ *	send_ordered_clustermsg: Send ordered message to all cluster members.
+ */
+	int		(*send_ordered_clustermsg)(ll_cluster_t*
+,			struct ha_msg* msg);
+/*
+ *	send_ordered_nodemsg:	Send ordered message to node.
+ */
+	int		(*send_ordered_nodemsg)(ll_cluster_t*
+,			struct ha_msg* msg
+,			const char* nodename);
+ 
+ 
 /*
  *	inputfd:	Return fd which can be given to select(2) or poll(2)
  *			for determining when messages are ready to be read.
  *			Only to be used in select() or poll(), please...
+ *			Note that due to IPC input buffering, always check
+ *			msgready() before going into select() or poll()
+ *			or you might hang there forever.
  */
 	int		(*inputfd)(ll_cluster_t*);
+ 
+/*
+ *	ipcchan:	Return IPC channel which can be given to
+ *			G_main_add_IPC_Channel() for mainloop use.
+ *			Please do not use send(), recv() directly.
+ *			Feel free to use waitin(), waitout(),
+ *			is_message_pending(), is_sending_blocked(),
+ *			set_recv_qlen(), set_send_qlen(), resume_io(),
+ *			verify_auth().
+ */
+	IPC_Channel*	(*ipcchan)(ll_cluster_t*);
 /*
  *	msgready:	Returns TRUE (1) when a message is ready to be read.
  */
@@ -227,7 +277,13 @@ struct llc_ops {
 #	define	LLC_FILTER_ALLHB	2
 #	define	LLC_FILTER_RAW		3
 
-	int (*setfmode)(ll_cluster_t*, int mode);
+	int (*setfmode)(ll_cluster_t*, unsigned mode);
+/*
+ *	Return the value of a heartbeat configuration parameter
+ *	as a malloc-ed string().  You need to free() the result when
+ *	you're done with it.
+ */
+	char * (*get_parameter)(ll_cluster_t *, const char * paramname);
 
 /*
  *	Return heartbeat's deadtime
@@ -253,10 +309,48 @@ struct llc_ops {
  */
 	int (*get_logfacility)(ll_cluster_t *);
 
+/*
+ *	Return the current resource ownership status.
+ *
+ *	NOTE:  this call will fail if heartbeat isn't
+ *	managing resources.  It can return "all", "local" or "foreign", "none"
+ *	or "transition".  This call will eventually go away when we rewrite
+ *	the resource management code.  "transition" means that things are
+ *	currently changing.
+ */
+	const char * (*get_resources)(ll_cluster_t *);
+
 
 	const char * (*errmsg)(ll_cluster_t*);
 };
 
+/* Parameters we can ask for via get_parameter */
+#define	KEY_HBVERSION	"hbversion"	/* Not a configuration parameter */
+#define	KEY_HOST	"node"
+#define KEY_HOPS	"hopfudge"
+#define KEY_KEEPALIVE	"keepalive"
+#define KEY_DEADTIME	"deadtime"
+#define KEY_DEADPING	"deadping"
+#define KEY_WARNTIME	"warntime"
+#define KEY_INITDEAD	"initdead"
+#define KEY_WATCHDOG	"watchdog"
+#define	KEY_BAUDRATE	"baud"
+#define	KEY_UDPPORT	"udpport"
+#define	KEY_FACILITY	"logfacility"
+#define	KEY_LOGFILE	"logfile"
+#define	KEY_DBGFILE	"debugfile"
+#define KEY_FAILBACK	"nice_failback"
+#define KEY_AUTOFAIL	"auto_failback"
+#define KEY_STONITH	"stonith"
+#define KEY_STONITHHOST "stonith_host"
+#define KEY_CLIENT_CHILD "respawn"
+#define KEY_RT_PRIO	"rtprio"
+#define KEY_GEN_METH	"hbgenmethod"
+#define KEY_REALTIME	"realtime"
+#define KEY_DEBUGLEVEL	"debug"
+#define KEY_NORMALPOLL	"normalpoll"
+#define KEY_APIPERM	"apiauth"
+#define KEY_MSGFMT      "msgfmt"
 
 ll_cluster_t*	ll_cluster_new(const char * llctype);
 #endif /* __HB_API_H */
