@@ -1,3 +1,4 @@
+/* $Id: pils.c,v 1.34.2.1 2004/04/20 09:08:50 lars Exp $ */
 /*
  * Copyright (C) 2001 Alan Robertson <alanr@unix.sh>
  * This software licensed under the GNU LGPL.
@@ -237,7 +238,8 @@ static PILPluginImports PILPluginImportSet =
 ,	PILLoadPlugin		/* load_plugin */
 ,	PILLog			/* Logging function */
 ,	g_malloc		/* Malloc function */
-,	g_free			/* free function */
+,	g_free			/* Free function */
+,	g_strdup		/* Strdup function */
 };
 
 static PIL_rc	ifmgr_register_interface(PILInterface* newif
@@ -429,7 +431,7 @@ RemoveAPILPlugin(PILPlugin*Plugin)
 		,	Plugin->plugin_name);
 	}
 	if (g_hash_table_lookup_extended(Pitype->Plugins
-	,	Plugin->plugin_name, &key, (void**)&Plugin)) {
+	,	Plugin->plugin_name, &key, (void*)&Plugin)) {
 
 		g_hash_table_remove(Pitype->Plugins, key);
 		RmAPILPlugin(key, Plugin, NULL);
@@ -542,7 +544,7 @@ RemoveAPILPluginType(PILPluginType*Plugintype)
 	PILPluginUniv*	Pluginuniv = Plugintype->piuniv;
 	gpointer	key;
 	if (g_hash_table_lookup_extended(Pluginuniv->PluginTypes
-	,	Plugintype->plugintype, &key, (void**)&Plugintype)) {
+	,	Plugintype->plugintype, &key, (void*)&Plugintype)) {
 
 		RmAPILPluginType(key, Plugintype, NULL);
 		g_hash_table_remove(Pluginuniv->PluginTypes, key);
@@ -1325,7 +1327,7 @@ PluginExists(const char * PluginPath)
 	/* Make sure we can read and execute the plugin file */
 	/* This test is nice, because dlopen reasons aren't return codes */
 
-	if (access(PluginPath, R_OK|X_OK) != 0) {
+	if (access(PluginPath, R_OK) != 0) {
 		if (DEBUGPLUGIN) {
 			PILLog(PIL_DEBUG, "Plugin file %s does not exist"
 			,	PluginPath);
@@ -1437,7 +1439,6 @@ PILLoadPlugin(PILPluginUniv* universe, const char * plugintype
 		,	PluginPath
 		,	lt_dlerror());
 		DELETE(PluginPath);
-		DelPILPluginType(pitype);
 		return PIL_NOPLUGIN;
 	}
 	DELETE(PluginPath);
@@ -1782,7 +1783,7 @@ RemoveAPILInterfaceType(PILInterfaceType*Iftype, PILInterfaceType* t2)
 		return;
 	}
 	if (g_hash_table_lookup_extended(Ifuniv->iftypes
-	,	Iftype->typename, &key, (gpointer*)&Iftype)) {
+	,	Iftype->typename, &key, (gpointer)&Iftype)) {
 
 		g_hash_table_remove(Ifuniv->iftypes, key);
 		RmAPILInterfaceType(key, Iftype, t2);
@@ -1868,7 +1869,7 @@ PILPluginTypeListPlugins(PILPluginType* pitype
 ,	int *		picount	/* Can be NULL ... */)
 {
 	const char *	piclass = pitype->plugintype;
-	int		plugincount = 0;
+	unsigned	plugincount = 0;
 	char **		result = NULL;
 	int		initoff = 0;
 	char **		pelem;
@@ -1902,6 +1903,7 @@ PILPluginTypeListPlugins(PILPluginType* pitype
 		if (dircount <= 0) {
 			if (files != NULL) {
 				FREE_DIRLIST(files, dircount);
+				files = NULL;
 			}
 			continue;
 		}
@@ -1917,7 +1919,7 @@ PILPluginTypeListPlugins(PILPluginType* pitype
 
 		for (j=0; j < dircount; ++j) {
 			char*	s;
-			int	slen = strlen(files[j]->d_name)
+			unsigned slen = strlen(files[j]->d_name)
 			-	STRLEN(PLUGINSUFFIX);
 
 			s = g_malloc(slen+1);
@@ -1926,6 +1928,7 @@ PILPluginTypeListPlugins(PILPluginType* pitype
 			result[initoff+j] = s;
 		}
 		FREE_DIRLIST(files, dircount);
+		files = NULL;
 	}
 
 	if (picount != NULL) {
