@@ -1,4 +1,4 @@
-static const char _bcast_Id [] = "$Id: bcast.c,v 1.31.2.1 2003/06/24 07:14:35 alan Exp $";
+/* $Id: bcast.c,v 1.33.2.2 2004/04/28 22:59:06 alan Exp $ */
 /*
  * bcast.c: UDP/IP broadcast-based communication code for heartbeat.
  *
@@ -66,8 +66,8 @@ static int		bcast_init(void);
 struct hb_media*	bcast_new(const char* interface);
 static int		bcast_open(struct hb_media* mp);
 static int		bcast_close(struct hb_media* mp);
-struct ha_msg*		bcast_read(struct hb_media* mp);
-static int		bcast_write(struct hb_media* mp, struct ha_msg* msg);
+static void*		bcast_read(struct hb_media* mp, int *lenp);
+static int		bcast_write(struct hb_media* mp, void* msg, int len);
 static int		bcast_make_receive_sock(struct hb_media* ei);
 static int		bcast_make_send_sock(struct hb_media * mp);
 static struct ip_private *
@@ -126,6 +126,7 @@ static void*			interfprivate;
 
 #define LOG	PluginImports->log
 #define MALLOC	PluginImports->alloc
+#define STRDUP  PluginImports->mstrdup
 #define FREE	PluginImports->mfree
 
 PIL_rc
@@ -158,24 +159,22 @@ PIL_PLUGIN_INIT(PILPlugin*us, const PILPluginImports* imports)
 
 static int 
 bcast_mtype(char** buffer) { 
-	
-	*buffer = MALLOC(sizeof(PIL_PLUGIN_S));
-
-	strcpy(*buffer, PIL_PLUGIN_S);
+	*buffer = STRDUP(PIL_PLUGIN_S);
+	if (!*buffer) {
+		return 0;
+	}
 
 	return STRLEN(PIL_PLUGIN_S);
 }
 
 static int
 bcast_descr(char **buffer) { 
+	*buffer = STRDUP("UDP/IP broadcast");
+	if (!*buffer) {
+		return 0;
+	}
 
-	const char* str = "UDP/IP broadcast";	
-
-	*buffer = MALLOC((strlen(str) * sizeof(char)) + 1);
-
-	strcpy(*buffer, str);
-
-	return strlen(str);
+	return strlen(*buffer);
 }
 
 static int
@@ -188,10 +187,6 @@ bcast_init(void)
 {
 	struct servent*	service;
 
-	(void)_heartbeat_h_Id;
-	(void)_bcast_Id;
-	(void)_ha_msg_h_Id;
-
 	g_assert(OurImports != NULL);
 
 	if (localudpport <= 0) {
@@ -247,11 +242,19 @@ bcast_new(const char * intf)
 		char * name;
 		memset(ret, 0, sizeof(*ret));
 		ret->pd = (void*)ipi;
-		name = MALLOC(strlen(intf)+1);
-		strcpy(name, intf);
-		ret->name = name;
+		name = STRDUP(intf);
+		if (name != NULL) {
+			ret->name = name;
+		} else {
+			FREE(ret);
+			ret = NULL;
+		}
+	}
+	if (ret != NULL) {
 		if (DEBUGPKT) {
-			PILCallLog(LOG, PIL_DEBUG, "bcast_new: returning ret (%s)", ret->name);
+			PILCallLog(LOG, PIL_DEBUG, 
+					"bcast_new: returning ret (%s)", 
+					ret->name);
 		}
 	}else{
 		FREE(ipi->interface);
@@ -323,18 +326,23 @@ bcast_close(struct hb_media* mp)
 
 	return(rc);
 }
+
+
+
+
 /*
  * Receive a heartbeat broadcast packet from BCAST interface
  */
 
-struct ha_msg *
-bcast_read(struct hb_media* mp)
+void *
+bcast_read(struct hb_media* mp, int * lenp)
 {
 	struct ip_private *	ei;
 	char			buf[MAXLINE];
 	int			addr_len = sizeof(struct sockaddr);
    	struct sockaddr_in	their_addr; /* connector's addr information */
 	int	numbytes;
+	void	*pkt;
 
 	BCASTASSERT(mp);
 	ei = (struct ip_private *) mp->pd;
@@ -342,7 +350,7 @@ bcast_read(struct hb_media* mp)
 	if (DEBUGPKT) {
 		PILCallLog(LOG, PIL_DEBUG
 		,	"bcast_read : reading from socket %d (writing to socket %d)"
-		,	ei->rsocket, ei->wsocket);
+			   ,	ei->rsocket, ei->wsocket);
 	}
 
 	if ((numbytes=recvfrom(ei->rsocket, buf, MAXLINE-1, MSG_WAITALL
@@ -362,37 +370,42 @@ bcast_read(struct hb_media* mp)
 		,	numbytes, inet_ntoa(their_addr.sin_addr));
 	}
 	if (DEBUGPKTCONT && numbytes > 0) {
-		PILCallLog(LOG, PIL_DEBUG, "%s", buf);
+	  PILCallLog(LOG, PIL_DEBUG, "%s", buf);
 	}
-	return(string2msg(buf, sizeof(buf)));
+	
+	pkt = ha_malloc(numbytes + 1);
+	if(!pkt){
+		PILCallLog(LOG, PIL_CRIT
+			    ,	"Error in allocating memory");
+		return(NULL);
+	}
+	
+	memcpy(pkt, buf, numbytes + 1);
+	*lenp = numbytes +1;
+	
+	return(pkt);
 }
 
+
 /*
  * Send a heartbeat packet over broadcast UDP/IP interface
  */
 
+
 static int
-bcast_write(struct hb_media* mp, struct ha_msg * msgptr)
+bcast_write(struct hb_media* mp, void *pkt, int len)
 {
 	struct ip_private *	ei;
 	int			rc;
-	char*			pkt;
-	int			size;
 
 	BCASTASSERT(mp);
 	ei = (struct ip_private *) mp->pd;
-
-	if ((pkt = msg2string(msgptr)) == NULL)  {
-		return(HA_FAIL);
-	}
-	size = strlen(pkt)+1;
-
-	if ((rc=sendto(ei->wsocket, pkt, size, 0
+	
+	if ((rc=sendto(ei->wsocket, pkt, len, 0
 	,	(struct sockaddr *)&ei->addr
-	,	sizeof(struct sockaddr))) != size) {
-		PILCallLog(LOG, PIL_CRIT, "Error sending packet: %s"
-		,	strerror(errno));
-		ha_free(pkt);
+	,	sizeof(struct sockaddr))) != len) {
+		PILCallLog(LOG, PIL_CRIT, "Unable to send bcast [%d] packet: %s"
+		,	rc, strerror(errno));
 		return(HA_FAIL);
 	}
 
@@ -403,12 +416,12 @@ bcast_write(struct hb_media* mp, struct ha_msg * msgptr)
    	}
 
 	if (DEBUGPKTCONT) {
-		PILCallLog(LOG, PIL_DEBUG, "bcast pkt out: [%s]", pkt);
+		PILCallLog(LOG, PIL_DEBUG, "bcast pkt out: [%s]", (char*)pkt);
    	}
-	ha_free(pkt);
 	return(HA_OK);
 }
 
+
 /*
  * Set up socket for sending broadcast UDP heartbeats
  */
@@ -648,12 +661,11 @@ new_ip_interface(const char * ifn, int port)
 
 	ep->bcast = broadaddr;
 
-	ep->interface = (char *)MALLOC(strlen(ifn)+1);
+	ep->interface = (char *)STRDUP(ifn);
 	if(ep->interface == NULL) {
 		FREE(ep);
 		return(NULL);
 	}
-	strcpy(ep->interface, ifn);
 	
 	memset(&ep->addr, 0, sizeof(ep->addr));	/* zero the struct */
 	ep->addr.sin_family = AF_INET;		/* host byte order */
@@ -724,10 +736,6 @@ if_get_broadaddr(const char *ifn, struct in_addr *broadaddr)
 	int		fd = -1;
 	struct ifreq ifr; /* points to one interface returned from ioctl */
 
-	/* get rid of compiler warnings about unreferenced variables */
-	(void)_heartbeat_h_Id;
-	(void)_ha_msg_h_Id;
-	
 	fd = socket (PF_INET, SOCK_DGRAM, 0);
 	
 	if (fd < 0) {
@@ -775,8 +783,32 @@ if_get_broadaddr(const char *ifn, struct in_addr *broadaddr)
 
 /*
  * $Log: bcast.c,v $
- * Revision 1.31.2.1  2003/06/24 07:14:35  alan
- * Fixed a format string.
+ * Revision 1.33.2.2  2004/04/28 22:59:06  alan
+ * Removed errorneous frees in error legs, and gave these error legs better diagnostics.
+ *
+ * Revision 1.36  2004/04/28 22:30:29  alan
+ * Put in some fixes for extra freeing of memory in the communications plugins.
+ *
+ * Revision 1.35  2004/03/03 05:31:50  alan
+ * Put in Gochun Shi's new netstrings on-the-wire data format code.
+ * this allows sending binary data, among many other things!
+ *
+ * Revision 1.34  2004/02/17 22:11:59  lars
+ * Pet peeve removal: _Id et al now gone, replaced with consistent Id header.
+ *
+ * Revision 1.33  2004/01/21 11:34:15  horms
+ * - Replaced numerous malloc + strcpy/strncpy invocations with strdup
+ *   * This usually makes the code a bit cleaner
+ *   * Also is easier not to make code with potential buffer over-runs
+ * - Added STRDUP to pils modules
+ * - Removed some spurious MALLOC and FREE redefinitions
+ *   _that could never be used_
+ * - Make sure the return value of strdup is honoured in error conditions
+ *
+ * Revision 1.32  2003/06/24 06:50:13  alan
+ * Added a missing format string.
+ * With the right level of debugging enabled, this could have allowed
+ * a hole to occur.
  *
  * Revision 1.31  2003/02/07 08:37:17  horms
  * Removed inclusion of portability.h from .h files
