@@ -1,3 +1,4 @@
+/* $Id: apcmastersnmp.c,v 1.8.2.1 2004/04/20 08:50:05 alan Exp $ */
 /*
  * Stonith module for APC Masterswitch (SNMP)
  * Copyright (c) 2001 Andreas Piesk <a.piesk@gmx.net>
@@ -30,6 +31,7 @@
 #include <netdb.h>
 #include <string.h>
 #include <unistd.h>
+#include <glib.h>
 
 #ifdef HAVE_NET_SNMP_NET_SNMP_CONFIG_H
 #       include <net-snmp/net-snmp-config.h>
@@ -114,6 +116,7 @@ static StonithImports*		OurImports;
 static void*			interfprivate;
 
 #define APC_MALLOC		PluginImports->alloc
+#define APC_STRDUP  		PluginImports->mstrdup
 #define APC_FREE		PluginImports->mfree
 
 PIL_rc
@@ -194,14 +197,6 @@ static const char *NOTapcID = "destroyed (APCMasterswitch)";
 
 #define _(text) dgettext(ST_TEXTDOMAIN, text)
 
-#ifndef APC_MALLOC
-#  define APC_MALLOC malloc
-#endif
-
-#ifndef APC_FREE
-#  define APC_FREE free
-#endif
-
 #ifndef APC_MALLOCT
 #  define APC_MALLOCT(t) ((t *)(APC_MALLOC(sizeof(t))))
 #endif
@@ -552,14 +547,12 @@ apcmastersnmp_hostlist(Stonith * s)
 				outlet_name);
 #endif
 		
-		if ((hl[num_outlets] = 
-				APC_MALLOC(strlen(outlet_name) + 1)) == NULL) {
+		if ((hl[num_outlets] = APC_STRDUP(outlet_name)) == NULL) {
 		    syslog(LOG_ERR, "%s: out of memory.", __FUNCTION__);
 		    apcmastersnmp_free_hostlist(hl);
 		    hl = NULL;
 		    return (hl);
 		}
-		strcpy(hl[num_outlets], outlet_name);
 		num_outlets++;
 	}
     }
@@ -654,6 +647,7 @@ apcmastersnmp_reset_req(Stonith * s, int request, const char *host)
 	}
 	
 	// found one
+	g_strdown(outlet_name);
 	if (strcmp(outlet_name, host) == 0) {
 #ifdef APC_DEBUG
 	    	syslog(LOG_DEBUG, "%s: Found %s at outlet: %i",
