@@ -1,3 +1,4 @@
+/* $Id: rcd_serial.c,v 1.11.2.1 2004/04/20 08:50:06 alan Exp $ */
 /*
  * Stonith module for RCD_SERIAL Stonith device
  *
@@ -46,6 +47,7 @@
 #	include <termio.h>
 #endif
 #include <sys/termios.h>
+#include <glib.h>
 
 #include <stonith/stonith.h>
 #include "stonith_signal.h"
@@ -117,6 +119,7 @@ static void*			interfprivate;
 
 #define LOG		PluginImports->log
 #define MALLOC		PluginImports->alloc
+#define STRDUP  	PluginImports->mstrdup
 #define FREE		PluginImports->mfree
 #define EXPECT_TOK	OurImports->ExpectToken
 #define STARTPROC	OurImports->StartProcess
@@ -231,12 +234,12 @@ RCD_alarm_handler(int sig) {
 	sa.sa_mask = sigmask;
 	sa.sa_flags = 0;
 	sigaction(SIGALRM, &sa, NULL);
+#endif
 
 #ifdef RCD_NOPAUSE
 	RCD_alarmcaught = 1;
 #endif
 	return;
-#endif
 }
 
 static int
@@ -277,12 +280,6 @@ static const char * NOTrcd_serialID = "Hey, dummy this has been destroyed (RCD_S
 	&& ((struct RCD_SerialDevice *)(i->pinfo))->RCD_SERIALid == RCD_SERIALid)
 
 
-#ifndef MALLOC
-#	define	MALLOC	malloc
-#endif
-#ifndef FREE
-#	define	FREE	free
-#endif
 #ifndef MALLOCT
 #	define     MALLOCT(t)      ((t *)(MALLOC(sizeof(t)))) 
 #endif
@@ -358,13 +355,12 @@ rcd_serial_hostlist(Stonith  *s)
 	memset(ret, 0, (rcd->hostcount+1)*sizeof(char*));
 
 	for (j=0; j < rcd->hostcount; ++j) {
-		ret[j] = MALLOC(strlen(rcd->hostlist[j])+1);
+		ret[j] = STRDUP(rcd->hostlist[j]);
 		if (ret[j] == NULL) {
 			rcd_serial_free_hostlist(ret);
 			ret = NULL;
 			return ret;
 		}
-		strcpy(ret[j], rcd->hostlist[j]);
 	}
 	return(ret);
 }
@@ -406,7 +402,7 @@ RCD_SERIAL_parse_config_info(struct RCD_SerialDevice* rcd, const char * info)
 	   duration of this function.
 	*/
 
-	copy = strdup(info);
+	copy = STRDUP(info);
 	if (!copy) {
 		syslog(LOG_ERR, "%s: out of memory!", __FUNCTION__);
 		return S_OOPS;
@@ -429,12 +425,13 @@ RCD_SERIAL_parse_config_info(struct RCD_SerialDevice* rcd, const char * info)
 	memset(rcd->hostlist, 0, 2*sizeof(char*));
 	rcd->hostcount = 0;
 
-	rcd->hostlist[0] = strdup(token);
+	rcd->hostlist[0] = STRDUP(token);
 	if (!rcd->hostlist[0]) {
 		syslog(LOG_ERR, "%s: out of memory!", __FUNCTION__);
 		ret = S_OOPS;
 		goto token_error;
 	}
+	g_strdown(rcd->hostlist[0]);
 	rcd->hostcount = 1;
 
 	/* Grab the device name */
@@ -446,7 +443,7 @@ RCD_SERIAL_parse_config_info(struct RCD_SerialDevice* rcd, const char * info)
 		goto token_error;
 	}
 
-	rcd->device = strdup(token);
+	rcd->device = STRDUP(token);
 	if (!rcd->device) {
 		syslog(LOG_ERR, "%s: out of memory!", __FUNCTION__);
 		ret = S_OOPS;
@@ -462,7 +459,7 @@ RCD_SERIAL_parse_config_info(struct RCD_SerialDevice* rcd, const char * info)
 		goto token_error;
 	}
 
-	rcd->signal = strdup(token);
+	rcd->signal = STRDUP(token);
 	if (!rcd->signal) {
 		syslog(LOG_ERR, "%s: out of memory!", __FUNCTION__);
 		ret = S_OOPS;
@@ -505,11 +502,11 @@ RCD_SERIAL_parse_config_info(struct RCD_SerialDevice* rcd, const char * info)
         /* free our private copy of the string we've been destructively
            parsing with strtok()
         */
-        free(copy);
+        FREE(copy);
         return S_OK;
 
 token_error:
-        free(copy);
+        FREE(copy);
         return(ret);
 }
 
@@ -525,7 +522,8 @@ rcd_serial_reset_req(Stonith * s, int request, const char * host)
 	int sigbit;
 	struct itimerval timer;
 	const char * err;
-
+	char* shost;
+	
 	if (!ISRCD_SERIALDEV(s)) {
 		syslog(LOG_ERR, "invalid argument to %s", __FUNCTION__);
 		return(S_OOPS);
@@ -534,11 +532,18 @@ rcd_serial_reset_req(Stonith * s, int request, const char * host)
 	rcd = (struct RCD_SerialDevice *) s->pinfo;
 
 	/* check that host matches */
+	if ((shost = STRDUP(host)) == NULL) {
+		syslog(LOG_ERR, "%s: strdup failed", __FUNCTION__);
+		return(S_OOPS);
+	}
+	g_strdown(shost);
 	if (strcmp(host, rcd->hostlist[0])) {
 		syslog(LOG_ERR, "%s: host '%s' not in hostlist.",
 			__FUNCTION__, host);
+		free(shost);
 		return(S_BADHOST);
 	}
+	free(shost);
 
 	/* Set the appropriate bit for the signal */
 	sigbit = *(rcd->signal)=='r' ? TIOCM_RTS : TIOCM_DTR;
