@@ -1,3 +1,4 @@
+/* $Id: wti_nps.c,v 1.10.2.1 2004/04/20 08:50:06 alan Exp $ */
 /*
  *
  *  Copyright 2001 Mission Critical Linux, Inc.
@@ -47,12 +48,6 @@
  *    private subnet.
  */
 
-
-/*
- * Version string that is filled in by CVS
- */
-static const char *version __attribute__ ((unused)) = "$Revision: 1.8.2.1 $"; 
-
 #include <portability.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -62,6 +57,7 @@ static const char *version __attribute__ ((unused)) = "$Revision: 1.8.2.1 $";
 #include <syslog.h>
 #include <libintl.h>
 #include <sys/wait.h>
+#include <glib.h>
 
 #include <stonith/stonith.h>
 
@@ -131,6 +127,7 @@ static void*			interfprivate;
 
 #define LOG		PluginImports->log
 #define MALLOC		PluginImports->alloc
+#define STRDUP  	PluginImports->mstrdup
 #define FREE		PluginImports->mfree
 #define EXPECT_TOK	OurImports->ExpectToken
 #define STARTPROC	OurImports->StartProcess
@@ -206,11 +203,9 @@ static const char * NOTnpsid = "Hey, dummy this has been destroyed (WTINPS)";
 				FREE(s);			\
 				(s)=NULL;			\
 			}					\
-			(s) = MALLOC(strlen(v)+1);		\
+			(s) = strdup(v);			\
 			if ((s) == NULL) {			\
 				syslog(LOG_ERR, _("out of memory"));\
-			}else{					\
-				strcpy((s),(v));		\
 			}					\
 			}
 
@@ -544,6 +539,7 @@ NPSNametoOutlet(struct WTINPS* nps, const char * name, char **outlets)
   					break;
   				}
   			}
+			g_strdown(sockname);
   			if (strcmp(name, sockname) == 0) {
   				ret = sockno;
   				sprintf(buf, "%d ", sockno);
@@ -655,12 +651,11 @@ wti_nps_hostlist(Stonith  *s)
 			if (numnames >= DIMOF(NameList)-1) {
 				break;
 			}
-			if ((nm = (char*)MALLOC(strlen(sockname)+1)) == NULL) {
+			if ((nm = strdup(sockname)) == NULL) {
 				syslog(LOG_ERR, "out of memory");
 				return(NULL);
 			}
-			memset(nm, 0, strlen(sockname)+1);
-			strcpy(nm, sockname);
+			g_strdown(nm);
 			NameList[numnames] = nm;
 			++numnames;
 			NameList[numnames] = NULL;
@@ -716,18 +711,16 @@ NPS_parse_config_info(struct WTINPS* nps, const char * info)
 	if (sscanf(info, "%s %[^\n\r\t]", dev, passwd) == 2
 	&&	strlen(passwd) > 1) {
 
-		if ((nps->device = (char *)MALLOC(strlen(dev)+1)) == NULL) {
+		if ((nps->device = strdup(dev)) == NULL) {
 			syslog(LOG_ERR, "out of memory");
 			return(S_OOPS);
 		}
-		if ((nps->passwd = (char *)MALLOC(strlen(passwd)+1)) == NULL) {
+		if ((nps->passwd = strdup(passwd)) == NULL) {
 			free(nps->device);
 			nps->device=NULL;
 			syslog(LOG_ERR, "out of memory");
 			return(S_OOPS);
 		}
-		strcpy(nps->device, dev);
-		strcpy(nps->passwd, passwd);
 		nps->config = 1;
 		return(S_OK);
 	}
@@ -778,10 +771,17 @@ wti_nps_reset_req(Stonith * s, int request, const char * host)
 		syslog(LOG_ERR, _("Cannot log into " DEVICE "."));
         }else{
 	        char *outlets;
+		char *shost;
 		int noutlet;
      
+		if ((shost = STRDUP(host)) == NULL) {
+			syslog(LOG_ERR, "strdup failed in NPS_reset_host");
+			return(S_OOPS);
+		}
+		g_strdown(shost);
 		noutlet = NPSNametoOutlet(nps, host, &outlets);
-		    
+		free(shost);
+
 		if (noutlet < 1) {
 			syslog(LOG_WARNING, _("%s %s "
 			"doesn't control host [%s]."), nps->idinfo
