@@ -1,3 +1,4 @@
+/* $Id: ccmlib_memapi.c,v 1.16.2.1 2004/04/20 05:05:35 alan Exp $ */
 /* 
  * ccmlib_memapi.c: Consensus Cluster Membership API
  *
@@ -82,9 +83,14 @@ typedef struct mbr_private_s {
 #define OC_EV_GET_NODEID(m,idx)  m->m_mem.m_array[idx].node_id
 #define OC_EV_GET_BORN(m,idx)  m->m_mem.m_array[idx].node_born_on
 
-#define OC_EV_COPY_NODE(m1,idx1,m2,idx2)  \
+#define OC_EV_COPY_NODE_WITHOUT_UNAME(m1,idx1,m2,idx2)  \
 		m1->m_mem.m_array[idx1]=m2->m_mem.m_array[idx2]
 
+#define OC_EV_COPY_NODE(m1,idx1,m2,idx2)  \
+		m1->m_mem.m_array[idx1]=m2->m_mem.m_array[idx2]; \
+		m1->m_mem.m_array[idx1].node_uname = \
+				strdup(m2->m_mem.m_array[idx2].node_uname)
+
 #define OC_EV_GET_SIZE(m)  m->m_size
 
 
@@ -96,6 +102,7 @@ void * cookie_get_data(void *ck);
 void * cookie_get_func(void *ck);
 void cookie_ref(void *ck);
 void cookie_unref(void *ck);
+static const char *llm_get_Id_from_Uuid(ccm_llm_t *stuff, uint uuid);
 
 
 
@@ -144,6 +151,7 @@ init_bornon(mbr_private_t *private,
 	return;
 }
 
+
 static void
 reset_bornon(mbr_private_t *private)
 {
@@ -158,7 +166,6 @@ reset_llm(mbr_private_t *private)
 	private->llm = NULL;
 }
 
-
 static int
 init_llmborn(mbr_private_t *private)
 {
@@ -193,7 +200,7 @@ init_llmborn(mbr_private_t *private)
 		}
 		ret = ch->ops->recv(ch,&msg);
 		if(ret == IPC_BROKEN) {
-			perror("connection denied");
+			fprintf(stderr, "connection denied\n");
 			return -1;
 		}
 	 	if(ret == IPC_FAIL){
@@ -262,6 +269,19 @@ compare(const void *value1, const void *value2)
 	return 0;
 }
 
+static const char *
+llm_get_Id_from_Uuid(ccm_llm_t *stuff, uint uuid)
+{
+	uint lpc = 0;
+	for (; lpc < stuff->n; lpc++) {
+		if(stuff->node[lpc].Uuid == uuid)
+			return stuff->node[lpc].Id;
+	}
+	return NULL;
+}
+
+
+
 static int
 get_new_membership(mbr_private_t *private,
 		ccm_meminfo_t *mbrinfo,
@@ -279,8 +299,6 @@ get_new_membership(mbr_private_t *private,
 
  	newmbr = *mbr = (mbr_track_t *) g_malloc(size+sizeof(int));
 
-
-
 	trans = OC_EV_SET_INSTANCE(newmbr,mbrinfo->trans);
 	n_members = OC_EV_SET_N_MEMBER(newmbr,mbrinfo->n);
 	OC_EV_SET_SIZE(newmbr, size);
@@ -288,12 +306,19 @@ get_new_membership(mbr_private_t *private,
 	j = OC_EV_SET_MEMB_IDX(newmbr,0);
 
 	for ( i = 0 ; i < n_members; i++ ) {
+		const char *uname = NULL;
+		gpointer	gborn; /* Help make gcc warning go away */
 		uuid =  CLLM_GET_UUID(private->llm, mbrinfo->member[i]);
 
+		uname = llm_get_Id_from_Uuid(private->llm, uuid);
+
+		newmbr->m_mem.m_array[j].node_uname = strdup(uname); 
+
 		OC_EV_SET_NODEID(newmbr,j,uuid);
 
-		born = GPOINTER_TO_INT(g_hash_table_lookup(private->bornon, 
-				GINT_TO_POINTER(mbrinfo->member[i])));
+		gborn = g_hash_table_lookup(private->bornon, 
+				GINT_TO_POINTER(mbrinfo->member[i]));
+		born = GPOINTER_TO_INT(gborn);
 
 		/* if there is already a born entry for the
 		 * node, use it. Otherwise create a born entry
@@ -322,7 +347,8 @@ get_new_membership(mbr_private_t *private,
 			if(!already_present(OC_EV_GET_NODEARRY(oldmbr),
 					OC_EV_GET_N_MEMBER(oldmbr),
 					OC_EV_GET_NODE(newmbr,i))){
-				OC_EV_COPY_NODE(newmbr, in_index, newmbr, i);
+				OC_EV_COPY_NODE_WITHOUT_UNAME(newmbr
+				,	in_index, newmbr, i);
 				in_index++;
 				OC_EV_INC_N_IN(newmbr);
 			}
@@ -348,9 +374,28 @@ get_new_membership(mbr_private_t *private,
 static void
 mem_free_func(void *data)
 {
+	unsigned lpc = 0;
+	char * uname;
 	mbr_track_t  *mbr_track =  (mbr_track_t *)data;
 
-	if(mbr_track) g_free(mbr_track);
+	if(mbr_track) {
+		/* free m_n_member uname, m_n_in is actually the same ptr */
+		for (lpc = 0 ; lpc < OC_EV_GET_N_MEMBER(mbr_track); lpc++ ) {
+			if ((uname = OC_EV_GET_NODE(mbr_track, lpc).node_uname))
+				g_free(uname);
+		}
+		/* free m_n_out uname */
+		for (lpc = OC_EV_GET_OUT_IDX(mbr_track)
+		;	lpc < OC_EV_GET_OUT_IDX(mbr_track)
+			+	OC_EV_GET_N_OUT(mbr_track)
+		;	lpc++) {
+
+			if ((uname = OC_EV_GET_NODE(mbr_track, lpc).node_uname))
+				g_free(uname);
+		}
+		g_free(mbr_track);
+	}
+	
 
 	return;
 }
@@ -496,7 +541,7 @@ mem_handle_event(class_t *class)
 			if (!private->special && !quorum){
 				update_bornons(private, mbr_track);
 				private->client_report = FALSE;
-				g_free(mbr_track);
+				mem_free_func(mbr_track);
 				break;
 			}
 			private->client_report = TRUE;
@@ -510,7 +555,7 @@ mem_handle_event(class_t *class)
 				/* we do not need the new mbr_track, 
 				*  the old one is the same as the new
 				*/
-				g_free(mbr_track);
+				mem_free_func(mbr_track);
 
 				oc_type = OC_EV_MS_PRIMARY_RESTORED;
 				cookie = private->cookie;
@@ -594,7 +639,7 @@ mem_handle_event(class_t *class)
 			reset_bornon(private);
 			cookie_unref(private->cookie);
 			private->cookie = NULL;
-		}
+ 		}
 	}
 	return TRUE;
 }
