@@ -1,3 +1,4 @@
+/* $Id: ccmmisc.c,v 1.15.2.1 2004/04/20 05:05:36 alan Exp $ */
 /* 
  * ccmmisc.c: Miscellaneous Consensus Cluster Service functions
  *
@@ -22,7 +23,6 @@
 #include <portability.h>
 #include <ccm.h>
 #include <stdlib.h>
-#include <ccm.h>
 #ifdef HAVE_MALLINFO
 #include <malloc.h>
 #endif
@@ -33,12 +33,9 @@
 int
 ccm_str2bitmap(const char *memlist, unsigned char **bitlist)
 {
-	size_t str_len =  strnlen(memlist, maxstrsz);
+	size_t str_len =  strlen(memlist);
 	int    outbytes = B64_maxbytelen(str_len);
 
-	(void)_heartbeat_h_Id; /* Make compiler happy */
-	(void)_ha_msg_h_Id; /* Make compiler happy */
-
 	if (str_len == 0) {
 	   	return bitmap_create(bitlist, MAXNODE);
 	}
