@@ -1,3 +1,4 @@
+/* $Id: apphbd.c,v 1.51.2.1 2004/04/20 05:05:37 alan Exp $ */
 /*
  * apphbd:	application heartbeat daemon
  *
@@ -83,6 +84,7 @@
 #include <clplumbing/cl_poll.h>
 #include <clplumbing/realtime.h>
 #include <clplumbing/uids.h>
+#include <clplumbing/cpulimits.h>
 #include <pils/generic.h>
 #include <pils/plugin.h>
 
@@ -178,12 +180,12 @@ static int set_notify_plugin(const char* option);
 static int set_debugfile(const char* option);
 static int set_logfile(const char* option);
 struct {
-	int	debug_level;
-	char	wdt_dev[MAXLINE];
-	int	wdt_interval_ms;
-	int	realtime;
-	char	debugfile[MAXLINE];
-	char	logfile[MAXLINE];
+	int		debug_level;
+	char		wdt_dev[MAXLINE];
+	int		wdt_interval_ms;
+	int		realtime;
+	char		debugfile[MAXLINE];
+	char		logfile[MAXLINE];
 } apphbd_config;
 
 struct directive{
@@ -235,17 +237,12 @@ apphb_dispatch(IPC_Channel* src, gpointer Client)
 		cl_log(LOG_DEBUG, "apphb_dispatch: client: %ld"
 		,	(long)client->pid);
 	}
-	/* FIXME!!!! ??? */
-	if (client->ch->ch_status == IPC_DISCONNECT) {
-		apphb_notify(client, APPHB_HUP);
-		client->deleteme = TRUE;
-		return FALSE;
-	}
 
 	while (!client->deleteme
 	&&	client->ch->ops->is_message_pending(client->ch)) {
 
 		if (client->ch->ch_status == IPC_DISCONNECT) {
+			apphb_notify(client, APPHB_HUP);
 			client->deleteme = TRUE;
 		}else{
 			if (!apphb_read_msg(client)) {
@@ -911,7 +908,7 @@ main(int argc, char ** argv)
 	}
 
 	if (argerr) {
-		usage(cmdname,LSB_EXIT_GENERIC);
+		usage(cmdname, LSB_EXIT_GENERIC);
 	}
 
 	if (req_status){
@@ -949,6 +946,14 @@ shutdown(int nsig)
 	}
 }
 
+static gboolean
+cpu_limit_timer(gpointer unused)
+{
+	(void)unused;
+	cl_cpu_limit_update();
+	return TRUE;
+}
+
 
 static int
 init_start()
@@ -981,6 +986,11 @@ init_start()
 		cl_disable_realtime();
 	}
 
+	if (!usenormalpoll) {
+		g_main_set_poll_func(cl_glibpoll);
+		ipc_set_pollfunc(cl_poll);
+	}
+
 	make_daemon();
 
 	/* Create a "waiting for connection" object */
@@ -999,15 +1009,17 @@ init_start()
 	/* Create a source to handle new connection requests */
 	G_main_add_IPC_WaitConnection(G_PRIORITY_HIGH, wconn
 	,	NULL, FALSE, apphb_new_dispatch, wconn, NULL);
-
-	if (!usenormalpoll) {
-		g_main_set_poll_func(cl_glibpoll);
+	if (apphbd_config.debug_level >= DBGMIN) {
+		int	ms_interval;
+		cl_cpu_limit_setpercent(20);
+		ms_interval = cl_cpu_limit_ms_interval();
+		Gmain_timeout_add(ms_interval, cpu_limit_timer, NULL);
 	}
 
 	/* Create the mainloop and run it... */
 	mainloop = g_main_new(FALSE);
 	cl_log(LOG_INFO, "Starting %s", cmdname);
-	cl_make_realtime(SCHED_RR, 5, 64);
+	cl_make_realtime(SCHED_RR, 5, 64, 64);
 	if (apphbd_config.wdt_dev[0] != EOS) {
 		open_watchdog(apphbd_config.wdt_dev);
 	}
